local allowed = {}
local warned = {}
local lastAllowedIncomingPlayer = nil

local blockMessage = "ShutUp: Your message has been blocked, since you are not in my friends list or my guild."

local function ShouldShow(name)
	return allowed[name] or ShutUpDB[name]
end

local orig = ChatFrame_MessageEventHandler
ChatFrame_MessageEventHandler = function(event, ...)
	if type(arg6) == "string" and arg6:find("GM") then return orig(event, ...) end
	if type(ShutUpDB) == "table" and type(arg1) == "string" and type(arg2) == "string" then
		local player = arg2
		local text = arg1
		if event == "CHAT_MSG_WHISPER" then
			if not ShouldShow(player) then
				if not warned[player] then
					SendChatMessage(blockMessage, "WHISPER", nil, player)
					warned[player] = true
				end
				if lastAllowedIncomingPlayer then
					ChatEdit_SetLastTellTarget(lastAllowedIncomingPlayer)
				end
				return
			else
				lastAllowedIncomingPlayer = player
			end
		elseif event == "CHAT_MSG_WHISPER_INFORM" then
			if text:find(blockMessage) then
				return
			end
			if not allowed[player] then
				ShutUpDB[player] = true
			end
		end
	end
	return orig(event, ...)
end

local frame = CreateFrame("Frame")
frame:SetScript("OnEvent", function()
	if event == "ADDON_LOADED" and arg1 == "ShutUp" then
		frame:UnregisterEvent("ADDON_LOADED")
		if type(ShutUpDB) ~= "table" then ShutUpDB = {} end

		-- Find all guild members.
		local offline = GetGuildRosterShowOffline()
		local selection = GetGuildRosterSelection()
		SetGuildRosterShowOffline(true)
		SetGuildRosterSelection(0)
		GetGuildRosterInfo(0)
		local numGuildMembers = GetNumGuildMembers()
		for i = 1, numGuildMembers, 1 do
			local name = GetGuildRosterInfo(i)
			if name then
				allowed[name] = true
			end
		end
		SetGuildRosterShowOffline(offline)
		SetGuildRosterSelection(selection)

		-- Find all friends
		local numFriends = GetNumFriends()

		for i = 1, numFriends do
			local name = GetFriendInfo(i)
			if name then
				allowed[name] = true
			end
		end
	elseif event == "PARTY_MEMBERS_CHANGED" then
		for i = 1, GetNumPartyMembers() do
			allowed[UnitName("party"..i)] = true
		end
	elseif event == "RAID_ROSTER_UPDATE" then
		for i = 1, GetNumRaidMembers() do
			allowed[UnitName("raid"..i)] = true
		end
	end
end)

frame:RegisterEvent("ADDON_LOADED")
frame:RegisterEvent("PARTY_MEMBERS_CHANGED")
frame:RegisterEvent("RAID_ROSTER_UPDATE")

